package probes::DNS;

=head1 NAME

probes::DNS - Name Service Probe for SmokePing

=head1 SYNOPSIS

 *** Probes ***
 + DNS
 binary = /usr/bin/dig

=head1 DESCRIPTION

Integrates dig as a probe into smokeping. The variable B<binary> must
point to your copy of the dig program. If it is not installed on
your system yet, you should install bind-utils >= 9.0.0.

The Probe asks the given host n-times for it's name. Where n is
the amount given in the Database section of the config File.

=head1 AUTHOR

Andre Stolze <stolze@uni-muenster.de>

=cut

use strict;
use base qw(probes::base);
use IPC::Open3;
use Symbol;
use Carp;

sub new($$$)
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = $class->SUPER::new(@_);

    # no need for this if we run as a cgi
    unless ( $ENV{SERVER_SOFTWARE} ) {
        
        croak "ERROR: DNS 'binary' not defined in FPing probe definition"
            unless defined $self->{properties}{binary};

        croak "ERROR: DNS 'binary' does not point to an executable"
            unless -f $self->{properties}{binary} and -x $self->{properties}{binary};
    
        my $return = `$self->{properties}{binary} L.ROOT-SERVERS.NET \@L.ROOT-SERVERS.NET 2>&1`;
        croak "ERROR: FPing must be installed setuid root or it will not work\n" 
            if $return =~ m/only.+root/;

        if ($return =~ m/;; Query time: ([0-9.]+)\smsec.*/s){
            $self->{pingfactor} = 1000;
            print "### dig seems to report in  miliseconds\n";
        } else {
            $self->{pingfactor} = 1000; # Gives us a good-guess default
            print "### assuming you are using an fping copy reporting in miliseconds\n";
        }
    };

    return $self;
}

sub ProbeDesc($){
    my $self = shift;
    return "DNS requests";
}

sub ping ($){
    my $self = shift;
    # do NOT call superclass ... the ping method MUST be overwriten
    my %upd;
    my $inh = gensym;
    my $outh = gensym;
    my $errh = gensym;
    # pinging nothing is pointless
    return unless @{$self->addresses};
    my @host = () ;
    my @addrs = $self->addresses;
    my @times = "$addrs[0][0]";
    push @times, ":";
    push @host, "$self->{properties}{binary} \@$addrs[0][0] $addrs[0][0] |grep \"Query time\"| gawk '{print \$4}'";
    for (my $run = 0; $run < $self->{cfg}{Database}{pings}; $run++){
      my $pid = open3($inh,$outh,$errh, @host);
      $self->{rtts}={};
      while (<$outh>){
        chomp;
	push @times, "$_";
      }
      waitpid $pid,0;
      close $errh;
      close $inh;
      close $outh;
    }
    my $ip = shift @times;
    next unless ':' eq shift @times; #drop the colon
    @times = map {sprintf "%.10e", $_ / $self->{pingfactor}} sort {$a <=> $b} grep {$_ ne "-"} @times;
    map { $self->{rtts}{$_} = [@times] } @{$self->{addrlookup}{$ip}} ;
    
}

1;
